@@ -1,6 +1,6 @@
 /* Datastore backend storing files
  *
- * Copyright (C) 2018 - 2019, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2018 - 2020, Stephan Mueller <smueller@chronox.de>
  *
  * License: see LICENSE file in root directory
  *
@@ -34,8 +34,9 @@
 #include <unistd.h>
 
 #include "acvpproxy.h"
-#include "logger.h"
 #include "internal.h"
+#include "json_wrapper.h"
+#include "logger.h"
 #include "request_helper.h"
 #include "threading_support.h"
 
@@ -51,7 +52,7 @@ static int
 acvp_datastore_write_data(const struct acvp_buf *data, const char *filename)
 {
 	FILE *file;
-	unsigned int written;
+	size_t written;
 	int ret = 0;
 
 	if (!data || !data->buf)
@@ -96,7 +97,7 @@ acvp_datastore_read_data(uint8_t **buf, size_t *buflen, const char *filename)
 	}
 
 	logger(LOGGER_DEBUG, LOGGER_C_DS_FILE, "Reading file %s\n", filename);
-	l_buflen = statbuf.st_size;
+	l_buflen = (size_t)statbuf.st_size;
 	l_buf = calloc(1, l_buflen + 1);
 	CKNULL(l_buf, -ENOMEM);
 
@@ -181,7 +182,7 @@ static int acvp_datastore_check_version(char *basedir, bool createdir)
 
 		snprintf(version, sizeof(version), "%d", ACVP_DS_VERSION);
 		writebuf.buf = (uint8_t *)version;
-		writebuf.len = strlen(version);
+		writebuf.len = (uint32_t)strlen(version);
 		CKINT(acvp_datastore_write_data(&writebuf, verfile));
 
 		return 0;
@@ -213,17 +214,12 @@ out:
 }
 
 static int
-acvp_datastore_file_testsessiondir(const struct acvp_testid_ctx *testid_ctx,
-				   char *pathname, size_t pathnamelen,
-				   bool createdir, bool secure_location)
+acvp_datastore_file_target_dir(const struct acvp_testid_ctx *testid_ctx,
+			       char *pathname, size_t pathnamelen,
+			       bool createdir, bool secure_location)
 {
 	const struct acvp_ctx *ctx = testid_ctx->ctx;
 	const struct acvp_datastore_ctx *datastore = &ctx->datastore;
-	const struct acvp_modinfo_ctx *modinfo = &ctx->modinfo;
-	const struct definition *def = testid_ctx->def;
-	const struct def_vendor *vendor = def->vendor;
-	const struct def_info *info = def->info;
-	const char *specificver = modinfo->specificver_filesafe;
 	int ret;
 	static atomic_t ds_ver_checked = ATOMIC_INIT(0);
 	static atomic_t ds_secure_ver_checked = ATOMIC_INIT(0);
@@ -253,6 +249,27 @@ acvp_datastore_file_testsessiondir(const struct acvp_testid_ctx *testid_ctx,
 
 	CKINT(acvp_datastore_file_dir(pathname, createdir));
 
+out:
+	return ret;
+}
+
+static int
+acvp_datastore_file_testsessiondir(const struct acvp_testid_ctx *testid_ctx,
+				   char *pathname, size_t pathnamelen,
+				   bool createdir, bool secure_location)
+{
+	const struct acvp_ctx *ctx = testid_ctx->ctx;
+	const struct acvp_modinfo_ctx *modinfo = &ctx->modinfo;
+	const struct definition *def = testid_ctx->def;
+	const struct def_vendor *vendor = def->vendor;
+	const struct def_info *info = def->info;
+	const char *specificver = modinfo->specificver_filesafe;
+	int ret;
+
+	CKINT(acvp_datastore_file_target_dir(testid_ctx, pathname,
+					     pathnamelen, createdir,
+					     secure_location));
+
 	if (vendor->vendor_name_filesafe) {
 		CKINT(acvp_extend_string(pathname, pathnamelen, "/%s",
 					 vendor->vendor_name_filesafe));
@@ -295,6 +312,98 @@ out:
 	return ret;
 }
 
+static int
+acvp_datastore_file_rename_version(const struct acvp_testid_ctx *testid_ctx,
+				   char *newversion)
+{
+	const struct definition *def = testid_ctx->def;
+	struct def_info *info = def->info;
+	char *currver = info->module_version_filesafe;
+	char pathname[FILENAME_MAX];
+	char newpathname[FILENAME_MAX];
+	int ret;
+
+	if (!info->module_version_filesafe)
+		return -EINVAL;
+
+	/* rename secure location */
+	CKINT(acvp_datastore_file_vectordir(testid_ctx, pathname,
+					    sizeof(pathname), false, true));
+	info->module_version_filesafe = newversion;
+	CKINT(acvp_datastore_file_vectordir(testid_ctx, newpathname,
+					    sizeof(pathname), true, true));
+	info->module_version_filesafe = currver;
+	ret = rename(pathname, newpathname);
+	if (ret) {
+		ret = -errno;
+		goto out;
+	}
+
+	/* rename regular location */
+	CKINT(acvp_datastore_file_vectordir(testid_ctx, pathname,
+					    sizeof(pathname), false, false));
+	info->module_version_filesafe = newversion;
+	CKINT(acvp_datastore_file_vectordir(testid_ctx, newpathname,
+					    sizeof(pathname), true, false));
+	info->module_version_filesafe = currver;
+	ret = rename(pathname, newpathname);
+	if (ret) {
+		ret = -errno;
+		goto out;
+	}
+
+
+out:
+	info->module_version_filesafe = currver;
+	return ret;
+}
+
+static int
+acvp_datastore_file_rename_name(const struct acvp_testid_ctx *testid_ctx,
+				char *newname)
+{
+	const struct definition *def = testid_ctx->def;
+	struct def_info *info = def->info;
+	char *currname = info->module_name_filesafe;
+	char pathname[FILENAME_MAX];
+	char newpathname[FILENAME_MAX];
+	int ret;
+
+	if (!info->module_name_filesafe)
+		return -EINVAL;
+
+	/* rename secure location */
+	CKINT(acvp_datastore_file_vectordir(testid_ctx, pathname,
+					    sizeof(pathname), false, true));
+	info->module_name_filesafe = newname;
+	CKINT(acvp_datastore_file_vectordir(testid_ctx, newpathname,
+					    sizeof(pathname), true, true));
+	info->module_name_filesafe = currname;
+	ret = rename(pathname, newpathname);
+	if (ret) {
+		ret = -errno;
+		goto out;
+	}
+
+	/* rename regular location */
+	CKINT(acvp_datastore_file_vectordir(testid_ctx, pathname,
+					    sizeof(pathname), false, false));
+	info->module_name_filesafe = newname;
+	CKINT(acvp_datastore_file_vectordir(testid_ctx, newpathname,
+					    sizeof(pathname), true, false));
+	info->module_name_filesafe = currname;
+	ret = rename(pathname, newpathname);
+	if (ret) {
+		ret = -errno;
+		goto out;
+	}
+
+
+out:
+	info->module_name_filesafe = currname;
+	return ret;
+}
+
 static int
 acvp_datastore_file_vectordir_vsid(const struct acvp_vsid_ctx *vsid_ctx,
 				   char *pathname, size_t pathnamelen,
@@ -348,7 +457,7 @@ acvp_datastore_file_write_authtoken(const struct acvp_testid_ctx *testid_ctx)
 	snprintf(file, sizeof(file), "%s/%s",
 		 pathname, datastore->jwttokenfile);
 	tmp.buf = (uint8_t *)auth->jwt_token;
-	tmp.len = auth->jwt_token_len;
+	tmp.len = (uint32_t)auth->jwt_token_len;
 	ret = acvp_datastore_write_data(&tmp, file);
 	if (ret) {
 		/*
@@ -389,7 +498,7 @@ acvp_datastore_file_write_authtoken(const struct acvp_testid_ctx *testid_ctx)
 		 pathname, datastore->messagesizeconstraint);
 	snprintf(msgsize, sizeof(msgsize), "%u", auth->max_reg_msg_size);
 	tmp.buf = (uint8_t *)msgsize;
-	tmp.len = strlen(msgsize);
+	tmp.len = (uint32_t)strlen(msgsize);
 	CKINT(acvp_datastore_write_data(&tmp, file));
 
 out:
@@ -409,7 +518,7 @@ acvp_datastore_file_uint(const char *pathname, const char *filename,
 
 	if (!stat(file, &statbuf) && statbuf.st_size) {
 		size_t msgsize_len;
-		uint32_t msgsize_int;
+		unsigned long msgsize_int;
 		char *msgsize = NULL;
 
 		logger(LOGGER_DEBUG, LOGGER_C_DS_FILE,
@@ -424,13 +533,40 @@ acvp_datastore_file_uint(const char *pathname, const char *filename,
 		if (msgsize_int >= UINT_MAX)
 			*id = UINT_MAX;
 		else
-			*id = msgsize_int;
+			*id = (uint32_t)msgsize_int;
 	}
 
 out:
 	return ret;
 }
 
+static int
+acvp_datastore_process_certinfo(const char *pathname, const char *filename,
+				char **cert_no)
+{
+	struct stat statbuf;
+	struct json_object *certinfo = NULL;
+	int ret = 0;
+	char file[FILENAME_MAX];
+
+	snprintf(file, sizeof(file), "%s/%s", pathname, filename);
+	if (!stat(file, &statbuf) && statbuf.st_size) {
+		struct json_object *certdata, *certversion;
+		const char *valId;
+
+		CKINT(json_read_data(file, &certinfo));
+		CKINT(json_split_version(certinfo, &certdata,
+					 &certversion));
+		CKINT(json_get_string(certdata, "validationId", &valId));
+		CKINT(acvp_duplicate(cert_no, valId));
+	}
+
+out:
+	if (certinfo)
+		json_object_put(certinfo);
+	return ret;
+}
+
 static int
 acvp_datastore_file_read_authtoken(const struct acvp_testid_ctx *testid_ctx)
 {
@@ -501,7 +637,7 @@ acvp_datastore_file_read_authtoken(const struct acvp_testid_ctx *testid_ctx)
 	       "Maximum file size constraint %u\n",
 	       auth->max_reg_msg_size);
 
-	/* Get testsession certificate ID */
+	/* Get testsession certificate request ID */
 	auth->testsession_certificate_id = 0;
 	CKINT(acvp_datastore_file_uint(pathname,
 				       datastore->testsession_certificate_id,
@@ -510,6 +646,12 @@ acvp_datastore_file_read_authtoken(const struct acvp_testid_ctx *testid_ctx)
 	       "Test session certificate ID: %u\n",
 	       auth->testsession_certificate_id);
 
+	/* Get testsession certificate number */
+	auth->testsession_certificate_number = NULL;
+	CKINT(acvp_datastore_process_certinfo(pathname,
+				datastore->testsession_certificate_info,
+				&auth->testsession_certificate_number));
+
 out:
 	return ret;
 }
@@ -666,6 +808,279 @@ out:
 	return ret;
 }
 
+static int
+acvp_datastore_find_verdict(const struct acvp_datastore_ctx *datastore,
+			    struct acvp_test_verdict_status *verdict,
+			    char *verdict_dir, size_t verdict_dir_len)
+{
+	struct stat statbuf;
+	int ret;
+
+	CKINT(acvp_extend_string(verdict_dir, verdict_dir_len,
+				 "/%s", datastore->verdictfile));
+
+	/* Verdict file exists, return information to  */
+	if (!stat(verdict_dir, &statbuf)) {
+		ACVP_BUFFER_INIT(verdict_buf);
+		int fd;
+
+		/* Positive return code as this is no error */
+		if (!verdict)
+			return EEXIST;
+
+		fd = open(verdict_dir, O_RDONLY | O_CLOEXEC);
+		if (fd < 0) {
+			ret = -errno;
+
+			logger(LOGGER_WARN, LOGGER_C_DS_FILE,
+			       "Cannot open file %s (%d)\n", verdict_dir, ret);
+			goto out;
+		}
+
+		verdict_buf.buf = mmap(NULL, (size_t)statbuf.st_size, PROT_READ,
+				       MAP_SHARED, fd, 0);
+		if (verdict_buf.buf == MAP_FAILED) {
+			logger(LOGGER_WARN, LOGGER_C_DS_FILE,
+			       "Cannot mmap file %s\n", verdict_dir);
+			ret = -ENOMEM;
+			goto out;
+		}
+
+		verdict_buf.len = (uint32_t)statbuf.st_size;
+		ret = acvp_get_verdict_json(&verdict_buf, &verdict->verdict);
+
+		munmap(verdict_buf.buf, (size_t)statbuf.st_size);
+		close(fd);
+
+		if (ret) {
+			logger(LOGGER_WARN, LOGGER_C_ANY,
+			       "File %s does not contain valid verdict\n",
+			       verdict_dir);
+			/*
+			 * We are not stopping here and will not goto out,
+			 * since we will report that the ID is unverified.
+			 */
+		}
+
+		return 0;
+	}
+
+	/*
+	 * If we have a verdict to fill and we reach here, we have no verdict
+	 * file.
+	 */
+	if (verdict)
+		verdict->verdict = acvp_verdict_unknown;
+
+out:
+	return ret;
+}
+
+static int
+acvp_datastore_find_modinfo(const struct acvp_datastore_ctx *datastore,
+			    struct acvp_test_verdict_status *verdict,
+			    char *dir, size_t dir_len)
+{
+	struct stat statbuf;
+	int ret;
+
+	CKINT(acvp_extend_string(dir, dir_len, "/%s", datastore->vectorfile));
+
+	/* Verdict file exists, return information to  */
+	if (!stat(dir, &statbuf)) {
+		ACVP_BUFFER_INIT(buf);
+		int fd;
+
+		/* Positive return code as this is no error */
+		if (!verdict)
+			return EEXIST;
+
+		fd = open(dir, O_RDONLY | O_CLOEXEC);
+		if (fd < 0) {
+			ret = -errno;
+
+			logger(LOGGER_WARN, LOGGER_C_DS_FILE,
+			       "Cannot open file %s (%d)\n", dir, ret);
+			goto out;
+		}
+
+		buf.buf = mmap(NULL, (size_t)statbuf.st_size, PROT_READ,
+			       MAP_SHARED, fd, 0);
+		if (buf.buf == MAP_FAILED) {
+			logger(LOGGER_WARN, LOGGER_C_DS_FILE,
+			       "Cannot mmap file %s\n", buf);
+			ret = -ENOMEM;
+			goto out;
+		}
+
+		buf.len = (uint32_t)statbuf.st_size;
+		ret = acvp_get_algoinfo_json(&buf, verdict);
+
+		munmap(buf.buf, (size_t)statbuf.st_size);
+		close(fd);
+
+		if (ret) {
+			logger(LOGGER_WARN, LOGGER_C_ANY,
+			       "File %s does not contain valid cipher information\n",
+			       dir);
+			/*
+			 * We are not stopping here and will not goto out,
+			 * since we will report that the ID is unverified.
+			 */
+		}
+
+		return 0;
+	}
+
+out:
+	return ret;
+}
+
+static int
+acvp_datastore_find_testid_verdict(const struct acvp_testid_ctx *testid_ctx)
+{
+	const struct acvp_ctx *ctx;
+	const struct acvp_datastore_ctx *datastore;
+	char verdict_file[FILENAME_MAX];
+	int ret;
+
+	CKNULL_C_LOG(testid_ctx, -EINVAL, LOGGER_C_DS_FILE,
+		     "Data store backend exchange info missing\n");
+
+	ctx = testid_ctx->ctx;
+	datastore = &ctx->datastore;
+
+	ret = acvp_datastore_file_vectordir(testid_ctx, verdict_file,
+					    sizeof(verdict_file), false,
+					    false);
+	if (ret) {
+		/* If pathname does not exist, we ignore it. */
+		return 0;
+	} else {
+		CKINT(acvp_datastore_find_verdict(datastore, NULL, verdict_file,
+						  sizeof(verdict_file)));
+	}
+
+out:
+	return ret;
+}
+
+static int
+acvp_datastore_get_testid_verdict(struct acvp_testid_ctx *testid_ctx)
+{
+	const struct acvp_ctx *ctx;
+	const struct acvp_datastore_ctx *datastore;
+	char vector_dir[FILENAME_MAX];
+	int ret;
+
+	CKNULL_C_LOG(testid_ctx, -EINVAL, LOGGER_C_DS_FILE,
+		     "Data store backend exchange info missing\n");
+
+	ctx = testid_ctx->ctx;
+	datastore = &ctx->datastore;
+
+	ret = acvp_datastore_file_vectordir(testid_ctx, vector_dir,
+					    sizeof(vector_dir), false,
+					    false);
+	if (ret) {
+		/* If pathname does not exist, we ignore it. */
+		return 0;
+	} else {
+		CKINT(acvp_datastore_find_verdict(datastore,
+						  &testid_ctx->verdict,
+						  vector_dir,
+						  sizeof(vector_dir)));
+
+		CKINT(acvp_datastore_file_vectordir(testid_ctx, vector_dir,
+						    sizeof(vector_dir), false,
+						    false));
+		CKINT(acvp_datastore_find_modinfo(datastore,
+						  &testid_ctx->verdict,
+						  vector_dir,
+						  sizeof(vector_dir)));
+	}
+
+out:
+	return ret;
+}
+
+static int
+acvp_datastore_find_vsid_verdict(struct acvp_vsid_ctx *vsid_ctx)
+{
+	const struct acvp_testid_ctx *testid_ctx;
+	const struct acvp_ctx *ctx;
+	const struct acvp_datastore_ctx *datastore;
+	char vector_dir[FILENAME_MAX];
+	int ret;
+
+	CKNULL_C_LOG(vsid_ctx, -EINVAL, LOGGER_C_DS_FILE,
+		     "Data store backend exchange info missing\n");
+	testid_ctx = vsid_ctx->testid_ctx;
+
+	CKNULL_C_LOG(testid_ctx, -EINVAL, LOGGER_C_DS_FILE,
+		     "Data store backend exchange info missing\n");
+
+	ctx = testid_ctx->ctx;
+	datastore = &ctx->datastore;
+
+	ret = acvp_datastore_file_vectordir_vsid(vsid_ctx, vector_dir,
+						 sizeof(vector_dir), false,
+						 false);
+	if (ret) {
+		/* If pathname does not exist, we ignore it. */
+		return 0;
+	} else {
+		CKINT(acvp_datastore_find_verdict(datastore, NULL, vector_dir,
+						  sizeof(vector_dir)));
+
+		CKINT(acvp_datastore_file_vectordir_vsid(vsid_ctx, vector_dir,
+							 sizeof(vector_dir),
+							 false, false));
+		CKINT(acvp_datastore_find_modinfo(datastore,
+						  &vsid_ctx->verdict,
+						  vector_dir,
+						  sizeof(vector_dir)));
+	}
+
+out:
+	return ret;
+}
+
+static int
+acvp_datastore_get_vsid_verdict(struct acvp_vsid_ctx *vsid_ctx)
+{
+	const struct acvp_testid_ctx *testid_ctx;
+	const struct acvp_ctx *ctx;
+	const struct acvp_datastore_ctx *datastore;
+	char verdict_file[FILENAME_MAX];
+	int ret;
+
+	CKNULL_C_LOG(vsid_ctx, -EINVAL, LOGGER_C_DS_FILE,
+		     "Data store backend exchange info missing\n");
+	testid_ctx = vsid_ctx->testid_ctx;
+
+	CKNULL_C_LOG(testid_ctx, -EINVAL, LOGGER_C_DS_FILE,
+		     "Data store backend exchange info missing\n");
+
+	ctx = testid_ctx->ctx;
+	datastore = &ctx->datastore;
+
+	ret = acvp_datastore_file_vectordir_vsid(vsid_ctx, verdict_file,
+						 sizeof(verdict_file), false,
+						 false);
+	if (ret) {
+		/* If pathname does not exist, we ignore it. */
+		return 0;
+	} else {
+		CKINT(acvp_datastore_find_verdict(datastore, &vsid_ctx->verdict,
+						  verdict_file,
+						  sizeof(verdict_file)));
+	}
+
+out:
+	return ret;
+}
+
 static int acvp_datastore_process_vsid(struct acvp_vsid_ctx *vsid_ctx,
 				       const char *datastore_base,
 				       const char *secure_base,
@@ -758,10 +1173,6 @@ static int acvp_datastore_process_vsid(struct acvp_vsid_ctx *vsid_ctx,
 		if (!ctx_opts->resubmit_result) {
 			char verdict_file[FILENAME_MAX];
 
-			logger(LOGGER_VERBOSE, LOGGER_C_DS_FILE,
-			       "Skipping submission for vsID %u since it was submitted already (%s exists)\n",
-			       vsid_ctx->vsid, processedpath);
-
 			CKINT(acvp_datastore_file_vectordir_vsid(vsid_ctx,
 				verdict_file, sizeof(verdict_file), false,
 				false));
@@ -769,6 +1180,10 @@ static int acvp_datastore_process_vsid(struct acvp_vsid_ctx *vsid_ctx,
 						 sizeof(verdict_file),
 						"/%s", datastore->verdictfile));
 			if (stat(verdict_file, &statbuf)) {
+				logger(LOGGER_VERBOSE, LOGGER_C_DS_FILE,
+				       "Skipping submission for vsID %u since it was submitted already, but fetching verdict\n",
+				       vsid_ctx->vsid);
+
 				/*
 				 * Tell the callback to only download the
 				 * verdict file but not process any results.
@@ -779,9 +1194,23 @@ static int acvp_datastore_process_vsid(struct acvp_vsid_ctx *vsid_ctx,
 				 */
 				vsid_ctx->fetch_verdict = true;
 				return cb(vsid_ctx, NULL);
+			} else {
+				CKINT(acvp_datastore_get_vsid_verdict(vsid_ctx));
+
+				/*
+				 * If we have a verdict which shows
+				 * acvp_verdict_unreceived, then resubmit
+				 * as POST operation.
+				 */
+				if (vsid_ctx->verdict.verdict !=
+				    acvp_verdict_unreceived) {
+					logger(LOGGER_VERBOSE, LOGGER_C_DS_FILE,
+					       "Skipping submission for vsID %u since it was submitted already (%s exists)\n",
+					       vsid_ctx->vsid, processedpath);
+
+					return 0;
+				}
 			}
-		} else {
-			vsid_ctx->resubmit_result = true;
 		}
 	}
 
@@ -842,8 +1271,8 @@ static int acvp_datastore_process_vsid(struct acvp_vsid_ctx *vsid_ctx,
 			goto out;
 		}
 
-		resp_buf = mmap(NULL, statbuf.st_size, PROT_READ, MAP_SHARED,
-				fd, 0);
+		resp_buf = mmap(NULL, (size_t)statbuf.st_size, PROT_READ,
+				MAP_SHARED, fd, 0);
 		if (resp_buf == MAP_FAILED) {
 			logger(LOGGER_WARN, LOGGER_C_DS_FILE,
 			       "Cannot mmap file %s\n", resppath);
@@ -853,14 +1282,14 @@ static int acvp_datastore_process_vsid(struct acvp_vsid_ctx *vsid_ctx,
 		}
 
 		buf.buf = resp_buf;
-		buf.len = statbuf.st_size;
+		buf.len = (uint32_t)statbuf.st_size;
 
 		/* Process response file */
 		ret = cb(vsid_ctx, &buf);
-		munmap(resp_buf, statbuf.st_size);
+		munmap(resp_buf, (size_t)statbuf.st_size);
 		close(fd);
 
-		if (ret)
+		if (ret < 0)
 			goto out;
 
 		/* Create processed file */
@@ -914,206 +1343,6 @@ static int acvp_datastore_file_find_responses_thread(void *arg)
 }
 #endif
 
-static int
-acvp_datastore_find_verdict(const struct acvp_datastore_ctx *datastore,
-			    struct acvp_test_verdict_status *verdict,
-			    char *verdict_dir, size_t verdict_dir_len)
-{
-	struct stat statbuf;
-	int ret;
-
-	CKINT(acvp_extend_string(verdict_dir, verdict_dir_len,
-				 "/%s", datastore->verdictfile));
-
-	/* Verdict file exists, return information to  */
-	if (!stat(verdict_dir, &statbuf)) {
-		ACVP_BUFFER_INIT(verdict_buf);
-		int fd;
-		bool test_passed;
-
-		/* Positive return code as this is no error */
-		if (!verdict)
-			return EEXIST;
-
-		fd = open(verdict_dir, O_RDONLY | O_CLOEXEC);
-		if (fd < 0) {
-			ret = -errno;
-
-			logger(LOGGER_WARN, LOGGER_C_DS_FILE,
-			       "Cannot open file %s (%d)\n", verdict_dir, ret);
-			goto out;
-		}
-
-		verdict_buf.buf = mmap(NULL, statbuf.st_size, PROT_READ,
-				       MAP_SHARED, fd, 0);
-		if (verdict_buf.buf == MAP_FAILED) {
-			logger(LOGGER_WARN, LOGGER_C_DS_FILE,
-			       "Cannot mmap file %s\n", verdict_dir);
-			ret = -ENOMEM;
-			goto out;
-		}
-
-		verdict_buf.len = statbuf.st_size;
-		ret = acvp_get_verdict_json(&verdict_buf, &test_passed);
-
-		munmap(verdict_buf.buf, statbuf.st_size);
-		close(fd);
-
-		if (ret) {
-			logger(LOGGER_WARN, LOGGER_C_ANY,
-			       "File %s does not contain valid verdict\n",
-			       verdict_dir);
-			/*
-			 * We are not stopping here and will not goto out,
-			 * since we will report that the ID is unverified.
-			 */
-		}
-
-		acvp_store_verdict(verdict, test_passed);
-
-		return 0;
-	}
-
-	/*
-	 * If we have a verdict to fill and we reach here, we have no verdict
-	 * file.
-	 */
-	if (verdict)
-		verdict->verdict = acvp_verdict_unknown;
-
-out:
-	return ret;
-}
-
-static int
-acvp_datastore_find_testid_verdict(const struct acvp_testid_ctx *testid_ctx)
-{
-	const struct acvp_ctx *ctx;
-	const struct acvp_datastore_ctx *datastore;
-	char verdict_file[FILENAME_MAX];
-	int ret;
-
-	CKNULL_C_LOG(testid_ctx, -EINVAL, LOGGER_C_DS_FILE,
-		     "Data store backend exchange info missing\n");
-
-	ctx = testid_ctx->ctx;
-	datastore = &ctx->datastore;
-
-	ret = acvp_datastore_file_vectordir(testid_ctx, verdict_file,
-					    sizeof(verdict_file), false,
-					    false);
-	if (ret) {
-		/* If pathname does not exist, we ignore it. */
-		return 0;
-	} else {
-		CKINT(acvp_datastore_find_verdict(datastore, NULL, verdict_file,
-						  sizeof(verdict_file)));
-	}
-
-out:
-	return ret;
-}
-
-static int
-acvp_datastore_get_testid_verdict(struct acvp_testid_ctx *testid_ctx)
-{
-	const struct acvp_ctx *ctx;
-	const struct acvp_datastore_ctx *datastore;
-	char verdict_file[FILENAME_MAX];
-	int ret;
-
-	CKNULL_C_LOG(testid_ctx, -EINVAL, LOGGER_C_DS_FILE,
-		     "Data store backend exchange info missing\n");
-
-	ctx = testid_ctx->ctx;
-	datastore = &ctx->datastore;
-
-	ret = acvp_datastore_file_vectordir(testid_ctx, verdict_file,
-					    sizeof(verdict_file), false,
-					    false);
-	if (ret) {
-		/* If pathname does not exist, we ignore it. */
-		return 0;
-	} else {
-		CKINT(acvp_datastore_find_verdict(datastore,
-						  &testid_ctx->verdict,
-						  verdict_file,
-						  sizeof(verdict_file)));
-	}
-
-out:
-	return ret;
-}
-
-static int
-acvp_datastore_find_vsid_verdict(const struct acvp_vsid_ctx *vsid_ctx)
-{
-	const struct acvp_testid_ctx *testid_ctx;
-	const struct acvp_ctx *ctx;
-	const struct acvp_datastore_ctx *datastore;
-	char verdict_file[FILENAME_MAX];
-	int ret;
-
-	CKNULL_C_LOG(vsid_ctx, -EINVAL, LOGGER_C_DS_FILE,
-		     "Data store backend exchange info missing\n");
-	testid_ctx = vsid_ctx->testid_ctx;
-
-	CKNULL_C_LOG(testid_ctx, -EINVAL, LOGGER_C_DS_FILE,
-		     "Data store backend exchange info missing\n");
-
-	ctx = testid_ctx->ctx;
-	datastore = &ctx->datastore;
-
-	ret = acvp_datastore_file_vectordir_vsid(vsid_ctx, verdict_file,
-						 sizeof(verdict_file), false,
-						 false);
-	if (ret) {
-		/* If pathname does not exist, we ignore it. */
-		return 0;
-	} else {
-		CKINT(acvp_datastore_find_verdict(datastore, NULL, verdict_file,
-						  sizeof(verdict_file)));
-	}
-
-out:
-	return ret;
-}
-
-static int
-acvp_datastore_get_vsid_verdict(struct acvp_vsid_ctx *vsid_ctx)
-{
-	const struct acvp_testid_ctx *testid_ctx;
-	const struct acvp_ctx *ctx;
-	const struct acvp_datastore_ctx *datastore;
-	char verdict_file[FILENAME_MAX];
-	int ret;
-
-	CKNULL_C_LOG(vsid_ctx, -EINVAL, LOGGER_C_DS_FILE,
-		     "Data store backend exchange info missing\n");
-	testid_ctx = vsid_ctx->testid_ctx;
-
-	CKNULL_C_LOG(testid_ctx, -EINVAL, LOGGER_C_DS_FILE,
-		     "Data store backend exchange info missing\n");
-
-	ctx = testid_ctx->ctx;
-	datastore = &ctx->datastore;
-
-	ret = acvp_datastore_file_vectordir_vsid(vsid_ctx, verdict_file,
-						 sizeof(verdict_file), false,
-						 false);
-	if (ret) {
-		/* If pathname does not exist, we ignore it. */
-		return 0;
-	} else {
-		CKINT(acvp_datastore_find_verdict(datastore, &vsid_ctx->verdict,
-						  verdict_file,
-						  sizeof(verdict_file)));
-	}
-
-out:
-	return ret;
-}
-
 /*
  * The function is a safety measure to ensure there is no mismatch between
  * the module definition used to download the test vectors compared to the
@@ -1154,10 +1383,12 @@ acvp_datastore_file_find_responses(const struct acvp_testid_ctx *testid_ctx,
 {
 	const struct acvp_ctx *ctx;
 	const struct acvp_datastore_ctx *datastore;
+	const struct acvp_opts_ctx *opts;
 	const struct definition *def;
 	struct dirent *dirent;
 	DIR *dir = NULL;
 	char datastore_base[FILENAME_MAX - 100];
+	char base[FILENAME_MAX - 100];
 	char secure_base[FILENAME_MAX - 100];
 	int ret;
 
@@ -1166,6 +1397,7 @@ acvp_datastore_file_find_responses(const struct acvp_testid_ctx *testid_ctx,
 
 	ctx = testid_ctx->ctx;
 	datastore = &ctx->datastore;
+	opts = &ctx->options;
 	def = testid_ctx->def;
 
 	CKNULL_C_LOG(datastore, -EINVAL, LOGGER_C_DS_FILE,
@@ -1187,9 +1419,16 @@ acvp_datastore_file_find_responses(const struct acvp_testid_ctx *testid_ctx,
 	else if (ret)
 		return ret;
 
+	ret = acvp_datastore_file_vectordir(testid_ctx, base, sizeof(base),
+					    false, false);
+	if (ret == -ENOENT)
+		return 0;
+	else if (ret)
+		return ret;
+
 	ret = acvp_datastore_file_vectordir(testid_ctx, secure_base,
-					    sizeof(secure_base), false,
-					    true);
+					    sizeof(secure_base),
+					    true, false);
 	if (ret == -ENOENT)
 		return 0;
 	else if (ret)
@@ -1204,7 +1443,7 @@ acvp_datastore_file_find_responses(const struct acvp_testid_ctx *testid_ctx,
 	 *
 	 * In case we do not find a match, just disregard the current testID.
 	 */
-	if (acvp_def_check(testid_ctx, secure_base))
+	if (acvp_def_check(testid_ctx, base))
 		return 0;
 
 	dir = opendir(datastore_base);
@@ -1263,7 +1502,7 @@ acvp_datastore_file_find_responses(const struct acvp_testid_ctx *testid_ctx,
 		vsid_ctx = calloc(1, sizeof(*vsid_ctx));
 		CKNULL(vsid_ctx, -ENOMEM);
 
-		vsid_ctx->vsid = vsid_val;
+		vsid_ctx->vsid = (uint32_t)vsid_val;
 		vsid_ctx->testid_ctx = testid_ctx;
 		if (clock_gettime(CLOCK_REALTIME, &vsid_ctx->start)) {
 			ret = -errno;
@@ -1296,7 +1535,7 @@ acvp_datastore_file_find_responses(const struct acvp_testid_ctx *testid_ctx,
 
 #ifdef ACVP_USE_PTHREAD
 		/* Disable threading in DEBUG mode */
-		if (logger_get_verbosity(LOGGER_C_ANY) >= LOGGER_DEBUG) {
+		if (opts->threading_disabled) {
 			logger(LOGGER_DEBUG, LOGGER_C_DS_FILE,
 			       "Disable threading support\n");
 			ret = acvp_datastore_process_vsid(vsid_ctx,
@@ -1362,8 +1601,8 @@ acvp_datastore_file_find_testsession(const struct definition *def,
 	struct dirent *dirent;
 	DIR *dir = NULL;
 	char pathname[FILENAME_MAX - 100];
-	char secure_base[FILENAME_MAX - 100];
-	unsigned int i = 0;
+	char base[FILENAME_MAX - 100];
+	unsigned int tcount = 0;
 	int ret;
 
 	CKNULL_C_LOG(ctx, -EINVAL, LOGGER_C_DS_FILE,
@@ -1400,7 +1639,7 @@ acvp_datastore_file_find_testsession(const struct definition *def,
 	CKNULL(dir, -errno);
 
 	/* Iterate through test session directory and process files */
-	while ((i < *testid_count) &&
+	while ((tcount < *testid_count) &&
 	       (dirent = readdir(dir)) != NULL) {
 		const struct acvp_search_ctx *search = &datastore->search;
 		unsigned long testid = strtoul(dirent->d_name, NULL, 10);
@@ -1415,7 +1654,7 @@ acvp_datastore_file_find_testsession(const struct definition *def,
 			continue;
 
 		/* Fudge the testid_ctx */
-		testid_ctx.testid = testid;
+		testid_ctx.testid = (uint32_t)testid;
 
 		/*
 		 * If specific testID is requested, only return requested
@@ -1481,20 +1720,20 @@ acvp_datastore_file_find_testsession(const struct definition *def,
 		 * the testid pointing to one of the two OEs, still both
 		 * OEs would be returned if this check is not made.
 		 */
-		ret = acvp_datastore_file_vectordir(&testid_ctx, secure_base,
-						    sizeof(secure_base), false,
-						    true);
+		ret = acvp_datastore_file_vectordir(&testid_ctx, base,
+						    sizeof(base), false,
+						    false);
 		if (!ret) {
-			if (acvp_def_check(&testid_ctx, secure_base))
+			if (acvp_def_check(&testid_ctx, base))
 				continue;
 		}
 		ret = 0;
 
-		testids[i] = testid;
-		i++;
+		testids[tcount] = (uint32_t)testid;
+		tcount++;
 	}
 
-	*testid_count = i;
+	*testid_count = tcount;
 
 out:
 	if (dir)
@@ -1512,6 +1751,8 @@ static struct acvp_datastore_be acvp_datastore_file = {
 	&acvp_datastore_file_read_authtoken,
 	&acvp_datastore_get_testid_verdict,
 	&acvp_datastore_get_vsid_verdict,
+	&acvp_datastore_file_rename_version,
+	&acvp_datastore_file_rename_name,
 };
 
 ACVP_DEFINE_CONSTRUCTOR(acvp_datastore_init)
